#ifndef TEATOTAL_H
#define TEATOTAL_H
/*
 * TEA Total Copyright (c) Alex Holden <alex@linuxhacker.org> 2000.
 *
 * This code is public domain; you can do whatever you want with it, though I
 * would prefer you to contribute any bug fixes back to me if possible.
 * This software comes with NO WARRANTY WHATSOEVER, not even the implied
 * warranties of merchantability and fitness for a particular purpose.
 */

#include "arch.h"
#include "util.h"
#include "getarg.h"

#define TEA_HEADER "TEA"
#define HEADER_LEN 3

/* Must be divisible by 4 */
#define MAXPKTLEN 4096

#define ENC_TEA '1'
#define ENC_TEA_HUFF '2'

#ifdef BUILD_HUFFMAN
#include "huffman.h"
#endif

#ifdef BUILD_BASE64
#include "base64.h"
#endif

/* The packed attributes are needed to prevent gcc from aligning the structure
 * elements on certain architectures */
typedef struct {
	u8 hdr[3] __attribute__ ((packed));	/* "TEA" */
	u8 encoding __attribute__ ((packed));	/* Encoding type */
	u8 byte_order __attribute__ ((packed));	/* Byte order */
	u16 datalen __attribute__ ((packed));	/* Length of encoded data */
	u32 seq __attribute__ ((packed));	/* Encoded sequence number */
	u32 data[MAXPKTLEN] __attribute__ ((packed));	/* Encoded data */
} teapacket;

typedef struct {
	u32 k[4];
	u32 seq;
	teapacket *pkt;
	char *keyfile;
	char *inputfile;
	char *outputfile;
	int infd;
	int outfd;
	getarg_state *ga;
#ifdef BUILD_PPKEYS
	char *inkeypass;
	char *outkeypass;
#endif
#ifdef BUILD_HUFFMAN
	huffstate *huff;
#endif
#ifdef BUILD_BASE64
	b64state *b64;
#endif
	unsigned char program;
	unsigned int untea : 1;
	unsigned int first_block : 1;
#ifdef BUILD_HUFFMAN
	unsigned int compress : 1;
#endif
#ifdef BUILD_BASE64
	unsigned int base64 : 1;
	unsigned int fnspecified : 1;
#endif
#ifdef BUILD_PPKEYS
	unsigned int protkey : 1;
	unsigned int convkey : 1;
#endif
} teastate;

enum PROGRAMS {
	PROGRAM_UNKNOWN,
	PROGRAM_TEATOTAL,
	PROGRAM_TEA,
	PROGRAM_UNTEA,
	PROGRAM_TEA_KGEN
};
 
extern void usage(teastate *state);
extern void initialise(teastate *state, int argc, char *argv[]);

#endif
